// Module Name: grid.cpp
// Author:      Stephen Manley (smanley@nyx.net)
//
// Description:
// 
// This class implments a grid of tiles that the user can independantly
// select to be any color. This allows the user to define a colored 
// path of tiles which will later represent the maze.
// 
////////////////////////////////////////////////////////////////////////////

#ifndef TILE
#include "square.h"
#endif

#ifndef MAZEGRID
#include "grid.h"
#endif

#include "iostream.h"
#include "fstream.h"
#include "GL/gl.h"
#include "GL/glut.h"

// Default constructor. This contains the options which were being used to
// test the grid in this application. It also creates the tiles that 
// compose the spaces in the grid.
MazeGrid::MazeGrid()
{
	rGridColor = 0.5;
	gGridColor = 0.5;
	bGridColor = 0.5;

    rSelectColor = 0.0;
    bSelectColor = 1.0;
    gSelectColor = 0.0;

	rBackgroundColor = 0.0;
	bBackgroundColor = 0.0;
	gBackgroundColor = 0.0;

    gridSpacing = 2;
    tileSize = 1;

    xOrigin = 2;
    yOrigin = 3;

    xSize = 20;
    ySize = 20;

    filename = "./grid_storage.cpp";

    int i = 0;
    
	// Create all the maze tiles (width x height)
	for (int x=0; x<xSize; x++)
	{
	    for (int y=0; y<ySize; y++)
	    {
	       gridTiles[i] = new Tile(x+xOrigin, y+yOrigin, tileSize, 0., 0., 0.);
	       i++;
	    }
	}
}

MazeGrid::~MazeGrid()
{
}

MazeGrid::MazeGrid(char* filename, int x, int y, int sizeX, int sizeY, int space)
{
	this->filename = filename;
	
	xOrigin = x;
	yOrigin = y;
	xSize = sizeX;
	ySize = sizeY;
	gridSpacing = space;
	
}

// Draws the grid on the screen
void MazeGrid::draw()
{
   // Draw the tiles first
   int gridSize = xSize*ySize;
   
   for (int k=0; k<gridSize; k++)
       gridTiles[k]->draw();

   // Draw the grid out over the tiles to show subdivisions
   glColor3f(rGridColor, gGridColor, bGridColor);
 
   glBegin(GL_LINES);

      for (int x=0; x<=xSize; x++)
      {
         glVertex2d(x+gridSpacing,0+gridSpacing);
         glVertex2d(x+gridSpacing, xSize+gridSpacing);
      }

      for (int y=0; y<=ySize; y++)
	  {
	     glVertex2d(0+gridSpacing, y+gridSpacing);
	     glVertex2d(ySize+gridSpacing, y+gridSpacing);
	  }
      
   glEnd();
}

void MazeGrid::init()
{
}

// Check to see if the grid contains a point, and if so, which tile the
// click occured on. Return 0 if succeeded and -1 if a point was not
// found.
int MazeGrid::containsPoint(double worldX, double worldY)
{
	// Determine that the click was in acceptable bounds
	if (!((worldX > xOrigin) && (worldX < (xSize+gridSpacing)) && (worldY > yOrigin) && (worldY < ySize+gridSpacing)))
	   return -1;
	 
	// Send the message to each of the tiles to see who got clicked
    int gridSize = xSize * ySize;

	for (int i=0; i<gridSize; i++)
	{
	   if (gridTiles[i]->containsPoint(worldX, worldY) == 0)
	   {            
           activeTile = gridTiles[i];
	   
	       return 0;
	   }
	}
	
	return -1;
}

// Retrieves the selected tile from the grid array so that it's
// properties can be examined or manipulated.
Tile* MazeGrid::getTile(int x)
{
	Tile* selectedTile = gridTiles[x];

	return selectedTile;
}

// Saves te grid to disk.
void MazeGrid::save()
{
    fstream outFile("/data/projects/graphics/mazemaker/grid.dat", ios::out|ios::trunc);
    
	// Write out the grid X&Y dimensions
    outFile << xSize << endl;
    outFile << ySize << endl;

    double r,g,b;

	// Write out the colors of each tile (X*Y)
    for (int i=0; i<xSize*ySize; i++)
    {
        gridTiles[i]->getTileColor(r, g, b);
        outFile << r << endl;
        outFile << g << endl;
        outFile << b << endl;
    }

    outFile.close();
}

// Loads the grid from disk.
void MazeGrid::load()
{
    fstream inFile("/data/projects/graphics/mazemaker/grid.dat", ios::in);
    
    // Read in the grid dimensions
    inFile >> xSize;
    inFile >> ySize;

    double r,g,b;

	// Read in the colors of each tile (X*Y)
    for (int i=0; i<xSize*ySize; i++)
    {
        inFile >> r;
        inFile >> g;
        inFile >> b;

        gridTiles[i]->setTileColor(r, g, b);
    }

    // Redraw the grid - do this in main
    // this->draw();

    inFile.close();

}

void MazeGrid::clear()
{ 
    int size = xSize * ySize;

    for (int i=0; i<size; i++)
        gridTiles[i]->setTileColor(rBackgroundColor, gBackgroundColor, bBackgroundColor);
}

void MazeGrid::setActiveColor(double r, double g, double b)
{
	rSelectColor = r;
	gSelectColor = g;
	bSelectColor = b;
}

void MazeGrid::setBackgroundColor(double r, double g, double b)
{
	rBackgroundColor = r;
	gBackgroundColor = g;
	bBackgroundColor = b;	
}

void MazeGrid::setGridColor(double r, double g, double b)
{
    rGridColor = r;
    gGridColor = g;
    bGridColor = b;
}


Tile* MazeGrid::getActiveTile()
{
    return activeTile;
}

void MazeGrid::getActiveColor(double &r, double &g, double &b)
{
    r = rSelectColor;
    g = gSelectColor;
    b = bSelectColor;
}

void MazeGrid::getBackgroundColor(double &r, double &g, double &b)
{
    r = rBackgroundColor;
    g = gBackgroundColor;
    b = bBackgroundColor;
}
