// Module Name: maze.cpp
// Author:      Stephen Manley (smanley@nyx.net)
//
// Description:
// 
// This is the main program that contains the functionality for the GLUT
// windowing toolkit and ties the mazemaker classes together. The class
// instances are declared as globals to get around the C style definitions
// of the GLUT instances.
// 
// Usage:
// 
// Left clicking on the grid will draw that tile in the selected color. 
// The default color is black. To change that, select a new color from
// the menu on the side by clicking on the color you'd like to draw in.
// 
// To erase the color, right clicking on the tile will color it in with
// the default background color. (in this case, black)
// 
// The grid NOT is automatically saved and restored when you start. To
// save a grid, use the 's' key. To load a grid, use the 'l' key. You
// can save or load a grid at any time. To get rid of a maze, delete 
// the grid file from the current working directory.
// 
// The grid can be cleared at any time by pressing 'c'.
// 
////////////////////////////////////////////////////////////////////////////

#include <GL/gl.h>
#include <GL/glut.h>
#include <iostream.h>

#ifndef MAZEGRID
#include "grid.h"
#endif

#ifndef COLORMENU
#include "menu.h"
#endif

#ifndef TILE
#include "square.h"
#endif

// Global Scene Objects
MazeGrid* maze;
ColorMenu* menu;
Tile* tile;

// Projection and Window Information
double left   = 0.0;  
double right  = 30.0;
double top    = 0.0;
double bottom = 25.0;

// Height and width of the active window
int windowHeight, windowWidth;

// Default background color
double rBack = 0.0;
double gBack = 0.0;
double bBack = 0.0;

// Configure projection information
void init(void) 
{
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glShadeModel (GL_SMOOTH);

   // Initialize projection
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(left, right, top, bottom, -1,1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

// Redraw the whole scene
void display(void)
{
   glClear (GL_COLOR_BUFFER_BIT);
   glColor3f (rBack, gBack, bBack);

   // Draw the objects in the scene
   maze->draw();
   menu->draw();
  
   glFlush ();
}

// Reshape the window - save the parameters
void reshape (int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);

   windowWidth = w;
   windowHeight = h; 
}

// Keyboard handler
void keyboard(unsigned char key, int x, int y)
{
   switch (key) {
      case 27:
         exit(0);
         break;

      case 'l':
         maze->load();
         display(); // redraw
         break;   

      case 's':
         maze->save();
         break;   

      case 'c':
         maze->clear();
         display();
         break;
   }
}

// Used to unproject a mouse click onto the screen and then determine 
// what the X, Y, Z world coordinate of the most click is at the near 
// clipping plane. Note that this will change when using a perspective
// projection.
// 
// The mouse click is passed to the grid first, and then to the menu.
// If the click is handled by the grid, then the selected tile is 
// redrawn in it's new color. Otherwise, the active color is changed 
// to the color selected in the menu.
// 
// Note that this method is only called for left mouse clicks.
static GLint selectObject(GLint x, GLint y)
{
	GLdouble worldX, worldY, worldZ;

	GLdouble modelMatrix[16];
	GLdouble projMatrix[16];
	GLint viewport[4];

	glGetIntegerv(GL_VIEWPORT, viewport);
	glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
	glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);

	gluUnProject(x, windowHeight - y, 1.0, modelMatrix, projMatrix, viewport, &worldX, &worldY, &worldZ);
	
    if (maze->containsPoint(worldX, worldY) == 0)
    {
			// If the tile color == active color, toggle, else draw w/color
			double r, g, b;
			maze->getActiveColor(r, g, b);
        	maze->getActiveTile()->setTileColor(r, g, b);
           	maze->getActiveTile()->draw();

        return 0;
    }

    if (menu->containsPoint(worldX, worldY) == 0)
    {
        double r,g,b;
        menu->getActiveColor(r,g,b);
        maze->setActiveColor(r,g,b);
    
        return 0;
    }
    
    return -1;
}


// This method works like select object, except the objective is to determine
// if a click occured in the maze, and if so, draw the tile in the background
// color, effectively "erasing" it from the grid.
int eraseObject(int x, int y)
{
	GLdouble worldX, worldY, worldZ;

	GLdouble modelMatrix[16];
	GLdouble projMatrix[16];
	GLint viewport[4];

	glGetIntegerv(GL_VIEWPORT, viewport);
	glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
	glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);

	gluUnProject(x, windowHeight - y, 1.0, modelMatrix, projMatrix, viewport, &worldX, &worldY, &worldZ);
	
    if (maze->containsPoint(worldX, worldY) == 0)
    {
			// If the tile color == active color, toggle, else draw w/color
			double rTest, gTest, bTest;
			double r, g, b;
			
            maze->getActiveTile()->getTileColor(rTest, gTest, bTest);
			maze->getBackgroundColor(r, g, b);

            maze->getActiveTile()->setTileColor(r, g, b);
            maze->getActiveTile()->draw();
            
            return 0;
    }
    
    return -1;
}

// Snags mouse clicks
void mouse(int button, int state, int mouseX, int mouseY)
{
    GLint hit;

	if (button == GLUT_LEFT_BUTTON)
    	hit = selectObject((GLint)mouseX, (GLint)mouseY);

    if (button == GLUT_RIGHT_BUTTON)
        hit = eraseObject(mouseX, mouseY);

    //glutPostRedisplay();
}

int main(int argc, char** argv)
{
   // Make the playing field..
   maze = new MazeGrid();
   menu = new ColorMenu(25,25);

   // Glut init
   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
   glutInitWindowSize (500, 500); 
   glutInitWindowPosition (100, 100);
   glutCreateWindow ("Mazemaker");

   init ();

   // Callbacks
   glutDisplayFunc(display); 
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   glutMouseFunc(mouse);
   glutMainLoop();
   
   return 0;
}
